@@ -4,13 +4,14 @@ use 5.008_001;
 use strict;
 use warnings;
 
-our $VERSION = '1.0099_03';
+our $VERSION = '1.1005_01';
 
 use Carp              ();
 use Fcntl             ();
 use File::Spec        ();
 use Exporter          ();
 use Data::MessagePack ();
+use Scalar::Util      ();
 
 use Text::Xslate::Util qw(
     mark_raw unmark_raw
@@ -29,7 +30,7 @@ our @EXPORT_OK = qw(
     html_builder
 );
 
-my $BYTECODE_VERSION = '1.3';
+my $BYTECODE_VERSION = '1.4';
 
 # $bytecode_version + $fullpath + $compiler_and_parser_options
 my $XSLATE_MAGIC   = qq{xslate;$BYTECODE_VERSION;%s;%s;};
@@ -59,7 +60,6 @@ sub input_layer { ref($_[0]) ? $_[0]->{input_layer} : ':utf8' }
 package Text::Xslate::Engine;
 
 use Text::Xslate::Util qw(
-    import_from
     make_error
     dump
 );
@@ -70,6 +70,9 @@ BEGIN {
     my $dump_load = scalar($Text::Xslate::Util::DEBUG =~ /\b dump=load \b/xms);
     *_DUMP_LOAD = sub(){ $dump_load };
 
+    my $save_src = scalar($Text::Xslate::Util::DEBUG =~ /\b save_src \b/xms);
+    *_SAVE_SRC  = sub() { $save_src };
+
     *_ST_MTIME = sub() { 9 }; # see perldoc -f stat
 
     my $cache_dir = '.xslate_cache';
@@ -98,25 +101,31 @@ my %compiler_option = (
 );
 
 my %builtin = (
-    raw          => \&Text::Xslate::Util::mark_raw,
-    html         => \&Text::Xslate::Util::html_escape,
+    html_escape  => \&Text::Xslate::Util::html_escape,
     mark_raw     => \&Text::Xslate::Util::mark_raw,
     unmark_raw   => \&Text::Xslate::Util::unmark_raw,
-    uri          => \&Text::Xslate::Util::uri_escape,
+    uri_escape   => \&Text::Xslate::Util::uri_escape,
+
     is_array_ref => \&Text::Xslate::Util::is_array_ref,
     is_hash_ref  => \&Text::Xslate::Util::is_hash_ref,
 
     dump         => \&Text::Xslate::Util::dump,
+
+    # aliases
+    raw          => 'mark_raw',
+    html         => 'html_escape',
+    uri          => 'uri_escape',
 );
 
 sub default_functions { +{} } # overridable
 
 sub options { # overridable
+    my($self) = @_;
     return {
         # name       => default
         suffix       => '.tx',
         path         => ['.'],
-        input_layer  => ':utf8',
+        input_layer  => $self->input_layer,
         cache        => 1, # 0: not cached, 1: checks mtime, 2: always cached
         cache_dir    => _DEFAULT_CACHE_DIR,
         module       => undef,
@@ -159,29 +168,30 @@ sub new {
     ];
 
     # function
-    my %funcs;
-    $class->_merge_hash(\%funcs, $class->default_functions());
+    my %added_funcs;
+    $class->_merge_hash(\%added_funcs, $class->default_functions());
 
     # 'module' overrides default functions
     if(defined $args{module}) {
-        $class->_merge_hash(\%funcs, import_from(@{$args{module}}));
+        $class->_merge_hash(\%added_funcs,
+            Text::Xslate::Util::import_from(@{$args{module}}));
     }
 
     # 'function' overrides imported functons
-    $class->_merge_hash(\%funcs, $args{function});
+    $class->_merge_hash(\%added_funcs, $args{function});
 
-    # the following functions are not overridable
-    foreach my $name(keys %builtin) {
-        if(exists $funcs{$name}) {
-            warnings::warnif(redefine =>
-                "$class: You cannot redefine builtin function '$name',"
-                . " because it is embeded in the engine");
-        }
-        $funcs{$name} = $builtin{$name};
-    }
+    my %funcs = %builtin;
+    $class->_register_builtin_methods(\%funcs);
+    # user defined functions (added functions) can override builtins
+    $class->_merge_hash(\%funcs, \%added_funcs);
+
+    $class->_resolve_function_aliases(\%funcs);
 
     $args{function} = \%funcs;
 
+    # used for the magic token
+    $args{added_function_names} = [sort keys %added_funcs];
+
     # internal data
     $args{template} = {};
 
@@ -196,11 +206,41 @@ sub _merge_hash {
     return;
 }
 
-sub load_string { # for <string>
+sub _resolve_function_aliases {
+    my($self, $funcs) = @_;
+
+    foreach my $f(values %{$funcs}) {
+        my %seen; # to avoid infinate loops
+        while(!( ref($f) or Scalar::Util::looks_like_number($f) )) {
+            my $v = $funcs->{$f} or $self->_error(
+               "Cannot resolve a function alias '$f',"
+               . " which refers nothing",
+            );
+
+            if( ref($v) or Scalar::Util::looks_like_number($v) ) {
+                $f = $v;
+                last;
+            }
+            else {
+                $seen{$v}++ and $self->_error(
+                    "Cannot resolve a function alias '$f',"
+                    . " which makes circular references",
+                );
+            }
+        }
+    }
+
+    return;
+}
+
+sub load_string { # called in render_string()
     my($self, $string) = @_;
     if(not defined $string) {
         $self->_error("LoadError: Template string is not given");
     }
+    $self->note('  _load_string: %s', join '\n', split /\n/, $string)
+        if _DUMP_LOAD;
+    $self->{source}{'<string>'} = $string if _SAVE_SRC;
     $self->{string_buffer} = $string;
     my $asm = $self->compile($string);
     $self->_assemble($asm, '<string>', \$string, undef, undef);
@@ -220,7 +260,7 @@ sub find_file {
     my $orig_mtime;
     my $cache_mtime;
     foreach my $p(@{$self->{path}}) {
-        print STDOUT "  find_file: $p / $file ...\n" if _DUMP_LOAD;
+        $self->note("  find_file: %s / %s ...\n", $p, $file) if _DUMP_LOAD;
 
         my $path_id;
         if(ref $p eq 'HASH') { # virtual path
@@ -252,9 +292,11 @@ sub find_file {
         $self->_error("LoadError: Cannot find '$file' (path: @{$self->{path}})");
     }
 
-    print STDOUT "  find_file: $fullpath (", ($cache_mtime || 0), ")\n" if _DUMP_LOAD;
+    $self->note("  find_file: %s (%d)\n",
+        $fullpath, $cache_mtime || 0) if _DUMP_LOAD;
 
     return {
+        name        => ref($fullpath) ? $file : $fullpath,
         fullpath    => $fullpath,
         cachepath   => $cachepath,
 
@@ -269,7 +311,7 @@ sub load_file {
 
     local $self->{from_include} = $from_include;
 
-    print STDOUT "load_file($file)\n" if _DUMP_LOAD;
+    $self->note("load_file(%s)\n", $file) if _DUMP_LOAD;
 
     if($file eq '<string>') { # simply reload it
         return $self->load_string($self->{string_buffer});
@@ -306,6 +348,8 @@ sub _load_source {
     my $fullpath  = $fi->{fullpath};
     my $cachepath = $fi->{cachepath};
 
+    $self->note("  _load_source: try %s ...\n", $fullpath) if _DUMP_LOAD;
+
     # This routine is called when the cache is no longer valid (or not created yet)
     # so it should be ensured that the cache, if exists, does not exist
     if(-e $cachepath) {
@@ -314,9 +358,11 @@ sub _load_source {
     }
 
     my $source = $self->slurp($fullpath);
+    $self->{source}{$fi->{name}} = $source if _SAVE_SRC;
 
     my $asm = $self->compile($source,
         file => $fullpath,
+        name => $fi->{name},
     );
 
     if($self->{cache} >= 1) {
@@ -347,8 +393,8 @@ sub _load_source {
         }
     }
     if(_DUMP_LOAD) {
-        printf STDERR "  _load_source: cache(%s)\n",
-            defined $fi->{cache_mtime} ? $fi->{cache_mtime} : 'undef';
+        $self->note("  _load_source: cache(%s)\n",
+            defined $fi->{cache_mtime} ? $fi->{cache_mtime} : 'undef');
     }
 
     return $asm;
@@ -369,8 +415,8 @@ sub _load_compiled {
     # see also tx_load_template() in xs/Text-Xslate.xs
     if(!( defined($fi->{cache_mtime}) and $self->{cache} >= 1
             and $threshold >= $fi->{orig_mtime} )) {
-        printf "  _load_compiled: no fresh cache: %s, %s",
-            $threshold, Text::Xslate::Util::p($fi) if _DUMP_LOAD;
+        $self->note( "  _load_compiled: no fresh cache: %s, %s",
+            $threshold || 0, Text::Xslate::Util::p($fi) ) if _DUMP_LOAD;
         $fi->{cache_mtime} = undef;
         return undef;
     }
@@ -412,9 +458,9 @@ sub _load_compiled {
                 return undef; # purge the cache
             }
             if($dep_mtime > $threshold){
-                printf "  _load_compiled: %s(%s) is newer than %s(%s)\n",
+                $self->note("  _load_compiled: %s(%s) is newer than %s(%s)\n",
                     $c->[1],    scalar localtime($dep_mtime),
-                    $cachepath, scalar localtime($threshold)
+                    $cachepath, scalar localtime($threshold) )
                         if _DUMP_LOAD;
                 return undef; # purge the cache
             }
@@ -423,8 +469,8 @@ sub _load_compiled {
     }
 
     if(_DUMP_LOAD) {
-        printf STDERR "  _load_compiled: cache(%s)\n",
-            defined $fi->{cache_mtime} ? $fi->{cache_mtime} : 'undef';
+        $self->note("  _load_compiled: cache(%s)\n",
+            defined $fi->{cache_mtime} ? $fi->{cache_mtime} : 'undef');
     }
 
     return \@asm;
@@ -448,13 +494,15 @@ sub _magic_token {
         ref($self->{compiler}) || $self->{compiler},
         $self->_extract_options(\%parser_option),
         $self->_extract_options(\%compiler_option),
-        [sort keys %{ $self->{function} } ],
+        $self->{added_function_names},
     ]);
 
     if(ref $fullpath) { # ref to content string
         require 'Digest/MD5.pm';
         my $md5 = Digest::MD5->new();
-        $md5->add(${$fullpath});
+        my $s = ${$fullpath};
+        utf8::encode($s);
+        $md5->add($s);
         $fullpath = join ':', ref($fullpath), $md5->hexdigest();
     }
     return sprintf $XSLATE_MAGIC, $fullpath, $self->{serial_opt};
@@ -507,6 +555,10 @@ sub _error {
     die make_error(@_);
 }
 
+sub note {
+    my($self, @args) = @_;
+    printf STDERR @args;
+}
 
 package Text::Xslate;
 1;
@@ -518,7 +570,7 @@ Text::Xslate - Scalable template engine for Perl5
 
 =head1 VERSION
 
-This document describes Text::Xslate version 1.0099_03.
+This document describes Text::Xslate version 1.1005_01.
 
 =head1 SYNOPSIS
 
@@ -555,9 +607,14 @@ This document describes Text::Xslate version 1.0099_03.
 
 =head1 DESCRIPTION
 
-B<Text::Xslate> is a template engine, tuned for persistent applications,
+B<Xslate> is a template engine, tuned for persistent applications,
 safe as an HTML generator, and with rich features.
 
+There are a lot of template engines in CPAN, for example Template-Toolkit,
+Text::MicroTemplate, HTML::Template, and so on, but all of them have
+demerits at some points. This is why Xslate is developed and now it is
+well-honed as the standard template engine for web applications.
+
 The concept of Xslate is strongly influenced by Text::MicroTemplate
 and Template-Toolkit 2, but the central philosophy of Xslate is different
 from them. That is, the philosophy is B<sandboxing> that the template logic
@@ -621,9 +678,9 @@ but is more powerful.
 
 This mechanism is also called as template inheritance.
 
-=head3 Easy to enhance
+=head3 Easiness to enhance
 
-Xslate is highly extensible. You can add functions and methods to the template
+Xslate is ready to enhance. You can add functions and methods to the template
 engine and even add a new syntax via extending the parser.
 
 =head1 INTERFACE
@@ -812,31 +869,13 @@ option for HASH references which are cached as you expect:
     my $tx = Text::Xslate->new( path => \%vpath );
     print $tx->render('hello.tx', { lang => 'Xslate' });
 
+Note that I<$string> must be a text string, not a binary string.
+
 =head3 B<< $tx->load_file($file) :Void >>
 
 Loads I<$file> into memory for following C<render()>.
 Compiles and saves it as disk caches if needed.
 
-It is a good idea to load templates before applications fork.
-Here is an example to to load all the templates which is in a given path:
-
-    my $path = ...;
-    my $tx = Text::Xslate->new(
-        path      => [$path],
-        cache_dir =>  $path,
-    );
-
-    find sub {
-        if(/\.tx$/) {
-            my $file = $File::Find::name;
-            $file =~ s/\Q$path\E .//xsm; # fix path names
-            $tx->load_file($file);
-        }
-    }, $path;
-
-    # fork and render ...
-
-
 =head3 B<< Text::Xslate->current_engine :XslateEngine >>
 
 Returns the current Xslate engine while executing. Otherwise returns C<undef>.
@@ -1014,6 +1053,25 @@ Augment modifiers.
 
 Default arguments and named arguments for macros.
 
+=item *
+
+External macros.
+
+Just idea: in the new macro concept, macros and external templates will be
+the same in internals:
+
+    : macro foo($lang) { "Hello, " ~ $lang ~ " world!" }
+    : include foo { lang => 'Xslate' }
+    : # => 'Hello, Xslate world!'
+
+    : extern bar 'my/bar.tx';     # 'extern bar $file' is ok
+    : bar( value => 42 );         # calls an external template
+    : include bar { value => 42 } # ditto
+
+=item *
+
+Customization of the default escaping filter
+
 =back
 
 =cut
