@@ -3,11 +3,12 @@ package Text::Xslate::PP;
 use 5.008_001;
 use strict;
 
-our $VERSION = '1.0099_03';
+our $VERSION = '1.1005_01';
 
 BEGIN{
     $ENV{XSLATE} = ($ENV{XSLATE} || '') . '[pp]';
 }
+
 use Text::Xslate::Util qw(
     $DEBUG
     p
@@ -21,17 +22,16 @@ use Text::Xslate::PP::Const qw(:all);
 use Text::Xslate::PP::State;
 use Text::Xslate::PP::Type::Raw;
 use Text::Xslate::PP::Opcode;
-use Text::Xslate ();
+use Text::Xslate::PP::Method;
 
 use Scalar::Util ();
 use overload     ();
 use Carp         ();
 
+# it must be loaded dynamically
+require Text::Xslate;
 
 my $state_class = 'Text::Xslate::PP::Opcode';
-$VERSION eq $state_class->VERSION
-    or die "Version mismatch (" . __PACKAGE__ . "/$VERSION v.s. "
-         . "$state_class/" . $state_class->VERSION . ")";
 
 $VERSION =~ s/_//; # for developpers versions
 
@@ -67,6 +67,11 @@ our %html_escape = (
 );
 our $html_metachars = sprintf '[%s]', join '', map { quotemeta } keys %html_escape;
 
+sub _register_builtin_methods {
+    my($self, $funcs) = @_;
+    Text::Xslate::PP::Method::tx_register_builtin_methods($funcs);
+}
+
 #
 # public APIs
 #
@@ -98,7 +103,7 @@ sub render {
         Carp::croak( sprintf("Xslate: Template variables must be a HASH reference, not %s", $vars ) );
     }
 
-    my $st = tx_load_template( $self, $name, 0 );
+    my $st = $self->tx_load_template( $name, 0 );
 
     local $_orig_die_handler  = $SIG{__DIE__};
     local $_orig_warn_handler = $SIG{__WARN__};
@@ -634,7 +639,7 @@ Text::Xslate::PP - Yet another Text::Xslate runtime in pure Perl
 
 =head1 VERSION
 
-This document describes Text::Xslate::PP version 1.0099_03.
+This document describes Text::Xslate::PP version 1.1005_01.
 
 =head1 DESCRIPTION
 
